@@ -7,14 +7,14 @@ use Fcntl 'O_CREAT', 'O_RDWR', 'LOCK_EX', 'LOCK_SH', 'O_WRONLY', 'O_RDONLY';
 sub O_ACCMODE () { O_RDONLY | O_RDWR | O_WRONLY }
 
 
-$VERSION = "0.97_02";
+$VERSION = "0.96";
 my $DEFAULT_MEMORY_SIZE = 1<<21;    # 2 megabytes
 my $DEFAULT_AUTODEFER_THRESHHOLD = 3; # 3 records
 my $DEFAULT_AUTODEFER_FILELEN_THRESHHOLD = 65536; # 16 disk blocksful
 
 my %good_opt = map {$_ => 1, "-$_" => 1}
                  qw(memory dw_size mode recsep discipline 
-                    autodefer autochomp autodefer_threshhold concurrent);
+                    autodefer autochomp autodefer_threshhold);
 
 sub TIEARRAY {
   if (@_ % 2 != 0) {
@@ -33,10 +33,6 @@ sub TIEARRAY {
     }
   }
 
-  if ($opts{concurrent}) {
-    croak("$pack: concurrent access not supported yet\n");
-  }
-
   unless (defined $opts{memory}) {
     # default is the larger of the default cache size and the 
     # deferred-write buffer size (if specified)
@@ -93,7 +89,7 @@ sub TIEARRAY {
     unless (seek $file, 1, SEEK_SET) {
       croak "$pack: your filehandle does not appear to be seekable";
     }
-    seek $file, 0, SEEK_SET;    # put it back
+    seek $file, 0, SEEK_SET     # put it back
     $fh = $file;                # setting binmode is the user's problem
   } elsif (ref $file) {
     croak "usage: tie \@array, $pack, filename, [option => value]...";
@@ -656,7 +652,7 @@ sub _mtwrite {
       if (@_) {
         $unwritten = $self->_downcopy($data, $end, $_[1] - $end);
       } else {
-        # Make the file longer to accommodate the last segment that doesn'
+        # Make the file longer to accomodate the last segment that doesn'
         $unwritten = $self->_downcopy($data, $end);
       }
     }
@@ -701,8 +697,6 @@ sub _upcopy {
 # moving everything in the block forwards to make room.
 # Instead of writing the last length($data) bytes from the block
 # (because there isn't room for them any longer) return them.
-#
-# Undefined $len means 'until the end of the file'
 sub _downcopy {
   my $blocksize = 8192;
   my ($self, $data, $pos, $len) = @_;
@@ -713,21 +707,11 @@ sub _downcopy {
       : $len > $blocksize? $blocksize : $len;
     $self->_seekb($pos);
     read $fh, my($old), $readsize;
-    my $last_read_was_short = length($old) < $readsize;
     $data .= $old;
-    my $writable;
-    if ($last_read_was_short) {
-      # If last read was short, then $data now contains the entire rest
-      # of the file, so there's no need to write only one block of it
-      $writable = $data;
-      $data = "";
-    } else {
-      $writable = substr($data, 0, $readsize, "");
-    }
-    last if $writable eq "";
     $self->_seekb($pos);
+    my $writable = substr($data, 0, $readsize, "");
+    last if $writable eq "";
     $self->_write_record($writable);
-    last if $last_read_was_short && $data eq "";
     $len -= $readsize if defined $len;
     $pos += $readsize;
   }
@@ -2009,7 +1993,7 @@ Tie::File - Access the lines of a disk file via a Perl array
 
 =head1 SYNOPSIS
 
-	# This file documents Tie::File version 0.97
+	# This file documents Tie::File version 0.96
 	use Tie::File;
 
 	tie @array, 'Tie::File', filename or die ...;
@@ -2427,14 +2411,14 @@ C<-E<gt>autodefer()> recovers the current value of the autodefer setting.
 =head1 CONCURRENT ACCESS TO FILES
 
 Caching and deferred writing are inappropriate if you want the same
-file to be accessed simultaneously from more than one process.  Other
-optimizations performed internally by this module are also
-incompatible with concurrent access.  A future version of this module will
-support a C<concurrent =E<gt> 1> option that enables safe concurrent access.
+file to be accessed simultaneously from more than one process.  You
+will want to disable these features.  You should do that by including
+the C<memory =E<gt> 0> option in your C<tie> calls; this will inhibit
+caching and deferred writing.
 
-Previous versions of this documentation suggested using C<memory
-=E<gt> 0> for safe concurrent access.  This was mistaken.  Tie::File
-will not support safe concurrent access before version 0.98.
+You will also want to lock the file while reading or writing it.  You
+can use the C<-E<gt>flock> method for this.  A future version of this
+module may provide an 'autolocking' mode.
 
 =head1 CAVEATS
 
@@ -2532,7 +2516,7 @@ any news of importance, will be available at
 
 =head1 LICENSE
 
-C<Tie::File> version 0.97 is copyright (C) 2003 Mark Jason Dominus.
+C<Tie::File> version 0.96 is copyright (C) 2002 Mark Jason Dominus.
 
 This library is free software; you may redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -2549,8 +2533,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this library program; it should be in the file C<COPYING>.
-If not, write to the Free Software Foundation, Inc., 51 Franklin Street,
-Fifth Floor, Boston, MA  02110-1301, USA
+If not, write to the Free Software Foundation, Inc., 59 Temple Place,
+Suite 330, Boston, MA 02111 USA
 
 For licensing inquiries, contact the author at:
 
@@ -2560,7 +2544,7 @@ For licensing inquiries, contact the author at:
 
 =head1 WARRANTY
 
-C<Tie::File> version 0.97 comes with ABSOLUTELY NO WARRANTY.
+C<Tie::File> version 0.96 comes with ABSOLUTELY NO WARRANTY.
 For details, see the license.
 
 =head1 THANKS
@@ -2597,8 +2581,7 @@ Peter Scott /
 Peter Somu /
 Autrijus Tang (again) /
 Tels (again) /
-Juerd Waalboer /
-Todd Rinaldo
+Juerd Waalboer
 
 =head1 TODO
 
